﻿namespace TestProject;

/// <summary>
/// 가우스 난수기
/// </summary>
public class GaussianRandom
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 난수기
    /// </summary>
    private Random random;

    /// <summary>
    /// 이용 가능 여부
    /// </summary>
    private bool available;

    /// <summary>
    /// 다음 가수스 값
    /// </summary>
    private double nextGaussianValue;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - GaussianRandom()

    /// <summary>
    /// 생성자
    /// </summary>
    public GaussianRandom()
    {
        this.random = new Random();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 난수 값 구하기 - GetRandomValue()

    /// <summary>
    /// 난수 값 구하기
    /// </summary>
    /// <returns>난수 값</returns>
    public double GetRandomValue()
    {
        if(this.available)
        {
            this.available = false;

            return this.nextGaussianValue;
        }

        double u1 = this.random.NextDouble();
        double u2 = this.random.NextDouble();

        double temporaryValue1 = Math.Sqrt(-2.0 * Math.Log(u1));
        double temporaryValue2 = 2.0 * Math.PI * u2;

        this.nextGaussianValue = temporaryValue1 * Math.Sin(temporaryValue2);

        this.available = true;

        return temporaryValue1 * Math.Cos(temporaryValue2);
    }

    #endregion
    #region 난수 값 구하기 - GetRandomValue(mean, standardDeviation)

    /// <summary>
    /// 난수 값 구하기
    /// </summary>
    /// <param name="mean">평균</param>
    /// <param name="standardDeviation">표준 편차</param>
    /// <returns>난수 값</returns>
    public double GetRandomValue(double mean, double standardDeviation)
    {
        return mean + standardDeviation * GetRandomValue();
    }

    #endregion
    #region 난수 값 구하기 - GetRandomValue(standardDeviation)

    /// <summary>
    /// 난수 값 구하기
    /// </summary>
    /// <param name="standardDeviation">표준 편차</param>
    /// <returns>난수 값</returns>
    public double GetRandomValue(double standardDeviation)
    {
        return standardDeviation * GetRandomValue();
    }

    #endregion
}